// $Id: Event.h,v 1.18 2004/03/01 12:23:25 vern Exp $
//
// Copyright (c) 1995, 1996, 1997, 1998, 1999, 2002
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#ifndef event_h
#define event_h

#include "Expr.h"
#include "EventRegistry.h"
#include "Serializer.h"

class EventMgr;

class Event : public BroObj {
public:
	// Each Bro that may generate events gets an internally unique ID. This
	// is always LOCAL for the local Bro. For remote Bros, it gets assigned
	// by the RemoteSerializer.

	typedef unsigned short SourceID;
	static const unsigned short LOCAL = 0;

	Event(EventHandlerPtr handler, val_list* args,
		SourceID src = LOCAL, BroObj* obj = 0);
	~Event();

	void SetNext(Event* n)		{ next_event = n; }
	Event* NextEvent() const	{ return next_event; }

	SourceID Source()		{ return src; }

	void Describe(ODesc* d) const;

protected:
	friend class EventMgr;

	// This method is protected to make sure that everybody goes through
	// EventMgr::Dispatch().
	void Dispatch(bool no_remote = false)
		{
		if ( event_serializer )
			event_serializer->Serialize(handler->Name(), args);

		handler->Call(args, no_remote);
		if ( obj )
			// obj->EventDone();
			Unref(obj);
		}

	EventHandlerPtr handler;
	val_list* args;
	SourceID src;
	BroObj* obj;
	Event* next_event;
};

class EventMgr : public BroObj {
public:
	EventMgr();
	~EventMgr();

	void QueueEvent(EventHandlerPtr h, val_list* vl,
			Event::SourceID src = Event::LOCAL, BroObj* obj = 0)
		{
		if ( h )
			QueueEvent(new Event(h, vl, src, obj));
		}

	void Dispatch();

	void Dispatch(Event* event, bool no_remote = false)
		{
		current_src = event->Source();
		event->Dispatch(no_remote);
		Unref(event);
		}

	void Drain();

	int HasEvents() const	{ return head != 0; }

	// Returns the source ID of last raised event.
	Event::SourceID CurrentSource()	{ return current_src; }

	// Returns an event_source record describing the local Bro.
	RecordVal* GetLocalEventSourceVal();

	void Describe(ODesc* d) const;

protected:
	void QueueEvent(Event* event);

	Event* head;
	Event* tail;
	Event::SourceID current_src;
	RecordVal* src_val;
};

extern EventMgr mgr;
extern Func* no_handler;

extern int num_events_queued;
extern int num_events_dispatched;

#endif
