// $Id: SerialObj.cc,v 1.1 2004/03/01 12:17:30 vern Exp $

#include "SerialObj.h"
#include "Serializer.h"

TransientID::ID TransientID::counter = 0;

SerialObj::FactoryMap* SerialObj::factories = 0;;

SerialObj* SerialObj::Instantiate(SerialType type)
	{
	FactoryMap::iterator f = factories->find(type);
	if ( f != factories->end() )
		return (SerialObj*) (*f->second)();

	run_time(fmt("Unknown object type %d", type));
	return 0;
	}

void SerialObj::Register(SerialType type, FactoryFunc f)
	{
	if ( ! factories )
		factories = new FactoryMap;

	(*factories)[type] = f;
	}

bool SerialObj::Serialize(Serializer* s, SerialInfo* i, bool cache) const
	{
	if ( cache )
		{
		const TransientID* tid = GetTID();
		assert(tid);

		SerializationCache::PermanentID pid = s->Cache()->Lookup(*tid);
		if ( pid != SerializationCache::NONE )
			return s->Write(false) && s->Write(pid);

		pid = s->Cache()->Register(this, *tid);
		if ( ! (s->Write(true) && s->Write(pid)) )
			return false;
		}

	return DoSerialize(s, i, SER_NONE);
	}

SerialObj* SerialObj::Unserialize(Serializer* s, SerialType type, bool cache)
	{
	SerializationCache::PermanentID pid = SerializationCache::NONE;

	if ( cache )
		{
		bool full_obj;

		if ( ! (s->Read(&full_obj) && s->Read(&pid)) )
			return 0;

		if ( ! full_obj )
			{
			// FIXME: Yet another const_cast to check eventually...
			SerialObj* obj = const_cast<SerialObj *>(s->Cache()->Lookup(pid));
			if ( obj )
				{
				// BroObjs need to be ref'ed.
				if ( IsBroObj(type) )
					Ref((BroObj*) obj);

				return obj;
				}

			s->Error("unknown object referenced");
			return 0;
			}
		}

	uint16 stype;
	if ( ! s->Read(&stype) )
		return 0;

	SerialObj* obj = Instantiate(SerialType(stype));

	if ( ! obj )
		{
		s->Error("unknown object type");
		return 0;
		}

	const TransientID* tid = obj->GetTID();

	if ( cache )
		{
		assert(tid);
		s->Cache()->Register(obj, *tid, pid);
		}

	if ( ! obj->DoUnserialize(s) )
		return 0;

	if ( stype & 0x0f00 != type & 0xf00 )
		{
		s->Error("type mismatch");
		return 0;
		}

	return obj;
	}

bool SerialObj::DoSerialize(Serializer* s, SerialInfo* info,
				SerialType type) const
	{
	assert(type != SER_NONE);
	return s->Write((uint16)type);
	}

bool SerialObj::DoUnserialize(Serializer* s)
	{
	// Nothing to do.
	return true;
	}
