// $Id: StateAccess.cc,v 1.1 2004/03/01 12:17:30 vern Exp $

#include "Val.h"
#include "StateAccess.h"
#include "Serializer.h"
#include "Event.h"
#include "NetVar.h"

StateAccess::StateAccess(Opcode arg_opcode,
		const MutableVal* arg_target, const Val* arg_op1,
		const Val* arg_op2, const Val* arg_op3)
	{
	opcode = arg_opcode;
	target.val = const_cast<MutableVal*>(arg_target);
	target_type = TYPE_MVAL;
	op1.val = const_cast<Val*>(arg_op1);
	op1_type = TYPE_VAL;
	op2 = const_cast<Val*>(arg_op2);
	op3 = const_cast<Val*>(arg_op3);

	RefThem();
	}

StateAccess::StateAccess(Opcode arg_opcode,
		const ID* arg_target, const Val* arg_op1,
		const Val* arg_op2, const Val* arg_op3)
	{
	opcode = arg_opcode;
	target.id = const_cast<ID*>(arg_target);
	target_type = TYPE_ID;
	op1.val = const_cast<Val*>(arg_op1);
	op1_type = TYPE_VAL;
	op2 = const_cast<Val*>(arg_op2);
	op3 = const_cast<Val*>(arg_op3);

	RefThem();
	}

StateAccess::StateAccess(Opcode arg_opcode,
		const ID* arg_target, const HashKey* arg_op1,
		const Val* arg_op2, const Val* arg_op3)
	{
	opcode = arg_opcode;
	target.id = const_cast<ID*>(arg_target);
	target_type = TYPE_ID;
	op1.key = arg_op1;
	op1_type = TYPE_KEY;
	op2 = const_cast<Val*>(arg_op2);
	op3 = const_cast<Val*>(arg_op3);
	RefThem();
	}

StateAccess::StateAccess(Opcode arg_opcode,
		const MutableVal* arg_target, const HashKey* arg_op1,
		const Val* arg_op2, const Val* arg_op3)
	{
	opcode = arg_opcode;
	target.val = const_cast<MutableVal*>(arg_target);
	target_type = TYPE_MVAL;
	op1.key = arg_op1;
	op1_type = TYPE_KEY;
	op2 = const_cast<Val*>(arg_op2);
	op3 = const_cast<Val*>(arg_op3);

	RefThem();
	}

StateAccess::~StateAccess()
	{
	if ( target_type == TYPE_ID )
		Unref(target.id);
	else
		Unref(target.val);

	if ( op1_type == TYPE_VAL )
		Unref(op1.val);

	Unref(op2);
	Unref(op3);
	}

void StateAccess::RefThem()
	{
	if ( target_type == TYPE_ID )
		Ref(target.id);
	else
		Ref(target.val);

	if ( op1_type == TYPE_VAL && op1.val )
		Ref(op1.val);

	if ( op2 )
		Ref(op2);
	if ( op3 )
		Ref(op3);
	}

bool StateAccess::CheckOld(ID* id, Val* index, Val* should, Val* is)
	{
	if ( ! should && ! is )
		return true;

	// 'should == index' means that 'is' should be non-nil.
	if ( should == index && is )
		return true;

	if ( should && is && same_atomic_val(should, is) )
		return true;

	Val* arg1;
	Val* arg2;
	Val* arg3;

	if ( index )
		{
		ODesc d;
		d.SetShort();
		index->Describe(&d);
		arg1 = new StringVal(fmt("%s[%s]", id->Name(), d.Description()));
		}
	else
		arg1 = new StringVal(id->Name());

	if ( should )
		{
		ODesc d;
		d.SetShort();
		should->Describe(&d);
		arg2 = new StringVal(d.Description());
		}
	else
		arg2 = new StringVal("<none>");

	if ( is )
		{
		ODesc d;
		d.SetShort();
		is->Describe(&d);
		arg3 = new StringVal(d.Description());
		}
	else
		arg3 = new StringVal("<none>");

	val_list* args = new val_list;
	args->append(arg1);
	args->append(arg2);
	args->append(arg3);
	mgr.QueueEvent(remote_state_inconsistency, args);

	return false;
	}

bool StateAccess::CheckOldSet(ID* id, Val* index, bool should, bool is)
	{
	if ( should == is )
		return true;

	ODesc d;
	d.SetShort();
	index->Describe(&d);

	Val* arg1 = new StringVal(fmt("%s[%s]", id->Name(), d.Description()));
	Val* arg2 = new StringVal(should ? "set" : "not set");
	Val* arg3 = new StringVal(is ? "set" : "not set");

	val_list* args = new val_list;
	args->append(arg1);
	args->append(arg2);
	args->append(arg3);
	mgr.QueueEvent(remote_state_inconsistency, args);

	return false;
	}

void StateAccess::Replay()
	{
	// For simplicity we assume that we only replay unserialized accesses.
	assert(target_type == TYPE_ID && op1_type == TYPE_VAL);

	Val* v = target.id->ID_Val();
	TypeTag t = v->Type()->Tag();

	switch ( opcode ) {
	case OP_ASSIGN:
		assert(op1.val);
		// There mustn't be a direct assignment to a unique ID.
		assert(target.id->Name()[0] != '#');
		CheckOld(target.id, 0, op1.val, v);
		target.id->SetVal(op1.val->Ref());
		break;

	case OP_INCR:
		assert(op1.val && op2);
		if ( IsIntegral(t) )
			{
			// We derive the amount as difference between old
			// and new value.
			int amount =
				op1.val->CoerceToInt() - op2->CoerceToInt();

			target.id->SetVal(new Val(v->CoerceToInt() + amount, t),
						OP_INCR);
			}
		break;

	case OP_ASSIGN_IDX:
		assert(op1.val && op2);

		if ( t == TYPE_TABLE )
			{
			assert(op2);
			CheckOld(target.id, op1.val, op3,
					v->AsTableVal()->Lookup(op1.val));
			v->AsTableVal()->Assign(op1.val, op2 ? op2->Ref() : 0);
			}

		else if ( t == TYPE_RECORD )
			{
			const char* field = op1.val->AsString()->CheckString();
			int idx = v->Type()->AsRecordType()->FieldOffset(field);
			if ( idx >= 0 )
				{
				CheckOld(target.id, op1.val, op3,
					v->AsRecordVal()->Lookup(idx));
				v->AsRecordVal()->Assign(idx, op2 ?
							op2->Ref() : 0);
				}
			else
				run_time(fmt("access replay: unknown record field %s for assign", field));
			}
		break;

	case OP_INCR_IDX:
		{
		assert(op1.val && op2 && op3);

		// We derive the amount as the difference between old
		// and new value.
		int amount = op2->CoerceToInt() - op3->CoerceToInt();

		if ( t == TYPE_TABLE )
			{
			t = v->Type()->AsTableType()->YieldType()->Tag();
			Val* lookup_op1 = v->AsTableVal()->Lookup(op1.val);
			int delta = lookup_op1->CoerceToInt() + amount;
			Val* new_val = new Val(delta, t);
			v->AsTableVal()->Assign(op1.val, new_val, OP_INCR );
			}

		else if ( t == TYPE_RECORD )
			{
			const char* field = op1.val->AsString()->CheckString();
			int idx = v->Type()->AsRecordType()->FieldOffset(field);
			if ( idx >= 0 )
				{
				t = v->Type()->AsRecordType()->FieldType(idx)->Tag();
				Val* lookup_field =
					v->AsRecordVal()->Lookup(idx);
				int delta =
					lookup_field->CoerceToInt() + amount;
				Val* new_val = new Val(delta, t);
				v->AsRecordVal()->Assign(idx, new_val, OP_INCR);
				}
			else
				run_time(fmt("access replay: unknown record field %s for assign", field));
			}
		}
		break;

	case OP_ADD:
		assert(op1.val);
		if ( t == TYPE_TABLE )
			{
			CheckOldSet(target.id, op1.val, op2 != 0,
					v->AsTableVal()->Lookup(op1.val) != 0);
			v->AsTableVal()->Assign(op1.val, 0);
			}
		break;

	case OP_DEL:
		assert(op1.val);
		if ( t == TYPE_TABLE )
			{
			if ( v->Type()->AsTableType()->IsSet() )
				CheckOldSet(target.id, op1.val, op2 != 0,
					v->AsTableVal()->Lookup(op1.val) != 0);
			else
				CheckOld(target.id, op1.val, op2,
					v->AsTableVal()->Lookup(op1.val));

			v->AsTableVal()->Delete(op1.val);
			}
		break;

	case OP_EXPIRE:
		assert(op1.val);
		if ( t == TYPE_TABLE )
			// No old check for expire. It may have already
			// been deleted by ourselves.
			v->AsTableVal()->Delete(op1.val);
		break;

	case OP_PRINT:
		assert(op1.val);
		internal_error("access replay for print not implemented");
		break;

	default:
		internal_error("access replay: unknown opcode for StateAccess");
		break;
		}
	}

bool StateAccess::Serialize(Serializer* s) const
	{
	if ( ! s->Write(char(opcode)) )
		return false;

	const ID* id =
		target_type == TYPE_ID ? target.id : target.val->UniqueID();

	if ( ! s->Write(id->Name()) )
		 return false;

	SerialInfo no_incr;

	if ( op1_type == TYPE_KEY )
		{
		Val* index =
			id->ID_Val()->AsTableVal()->RecoverIndex(this->op1.key);

		if ( ! index )
			return false;
		if ( ! index->Serialize(s, true, no_incr) )
			return false;

		Unref(index);
		}

	else if ( ! op1.val->Serialize(s, true, no_incr) )
		return false;

	SERIALIZE_OPTIONAL_VAL(s, op2, true);
	SERIALIZE_OPTIONAL_VAL(s, op3, true);

	return true;
	}

StateAccess* StateAccess::Unserialize(Serializer* s)
	{
	StateAccess* sa = new StateAccess();

	char c;
	if ( ! s->Read(&c) )
		{
		delete sa;
		return 0;
		}

	sa->opcode = Opcode(c);

	const char* name;
	if ( ! s->Read(&name, 0) )
		{
		delete sa;
		return false;
		}

	sa->target_type = TYPE_ID;
	sa->target.id = global_scope()->Lookup(name);
	if ( ! sa->target.id )
		{
		s->Error("unknown id name");
		delete [] name;
		delete sa;
		return 0;
		}
	else
		Ref(sa->target.id);

	sa->op1_type = TYPE_VAL;
	sa->op1.val = Val::Unserialize(s, 0, TYPE_ANY);

	if ( ! sa->op1.val )
		{
		delete [] name;
		delete sa;
		return 0;
		}

	UNSERIALIZE_OPTIONAL_STATIC(s, sa->op2, Val::Unserialize(s, 0, TYPE_ANY), sa);
	UNSERIALIZE_OPTIONAL_STATIC(s, sa->op3, Val::Unserialize(s, 0, TYPE_ANY), sa);

	delete [] name;
	return sa;
	}

void StateAccess::Describe(ODesc* d) const
	{
	const ID* id;
	const char* unique = "";

	if ( target_type == TYPE_ID )
		{
		id = target.id;
		if ( id->ID_Val()->IsMutableVal() && id->Name()[0] != '#' )
			unique = fmt(" (%s)", id->ID_Val()->AsMutableVal()->UniqueID()->Name());
		}
	else
		id = target.val->UniqueID();

	const Val* op1 = op1_type == TYPE_VAL ?
		this->op1.val :
		id->ID_Val()->AsTableVal()->RecoverIndex(this->op1.key);

	switch ( opcode ) {
	case OP_ASSIGN:
		assert(op1);
		d->Add(id->Name());
		d->Add(unique);
		d->Add(" = ");
		op1->Describe(d);
		if ( op2 )
			{
			d->Add(" (");
			op2->Describe(d);
			d->Add(")");
			}
		break;

	case OP_INCR:
		assert(op1 && op2);
		d->Add(id->Name());
		d->Add(unique);
		d->Add(" += ");
		d->Add(op1->CoerceToInt() - op2->CoerceToInt());
		op1->Describe(d);
		break;

	case OP_ASSIGN_IDX:
		assert(op1 && op2);
		d->Add(id->Name());
		d->Add(unique);
		d->Add("[");
		op1->Describe(d);
		d->Add("]");
		d->Add(" = ");
		op2->Describe(d);
		if ( op3 )
			{
			d->Add(" (");
			op3->Describe(d);
			d->Add(")");
			}
		break;

	case OP_INCR_IDX:
		assert(op1 && op2 && op3);
		d->Add(id->Name());
		d->Add(unique);
		d->Add("[");
		op1->Describe(d);
		d->Add("]");
		d->Add(" += ");
		d->Add(op2->CoerceToInt() - op3->CoerceToInt());
		break;

	case OP_ADD:
		assert(op1);
		d->Add("add");
		d->Add(id->Name());
		d->Add(unique);
		d->Add("[");
		op1->Describe(d);
		d->Add("]");
		if ( op2 )
			{
			d->Add(" (");
			op2->Describe(d);
			d->Add(")");
			}
		break;

	case OP_DEL:
		assert(op1);
		d->Add("del");
		d->Add(id->Name());
		d->Add(unique);
		d->Add("[");
		op1->Describe(d);
		d->Add("]");
		if ( op2 )
			{
			d->Add(" (");
			op2->Describe(d);
			d->Add(")");
			}
		break;

	case OP_EXPIRE:
		assert(op1);
		d->Add("expire");
		d->Add(id->Name());
		d->Add(unique);
		d->Add("[");
		op1->Describe(d);
		d->Add("]");
		if ( op2 )
			{
			d->Add(" (");
			op2->Describe(d);
			d->Add(")");
			}
		break;

	case OP_PRINT:
		assert(op1);
		d->Add("print ");
		d->Add(id->Name());
		d->Add(unique);
		op1->Describe(d);
		break;

	default:
		internal_error("unknown opcode for StateAccess");
		break;
		}
	}
