// $Id: StateAccess.h,v 1.1 2004/03/01 12:17:30 vern Exp $
//
// A class describing a state-modyfing access to a Value or an ID.

#ifndef STATEACESSS_H
#define STATEACESSS_H

//#define DEBUG_STATE(s...)	fprintf(stderr, s);
#define DEBUG_STATE(s...)

class Val;
class ID;
class MutableVal;
class HashKey;
class ODesc;
class Serializer;

enum Opcode {	// Op1	Op2 Op3 (Vals)
	OP_ASSIGN,	// new	old
	OP_ASSIGN_IDX,	// new	old
	OP_ADD,		// idx  old
	OP_INCR,	// idx  new old
	OP_INCR_IDX,	// idx  new old
	OP_DEL,		// idx  old
	OP_PRINT,	// args
	OP_EXPIRE	// idx
};

class StateAccess {
public:
	StateAccess(Opcode opcode, const ID* target, const Val* op1,
			const Val* op2 = 0, const Val* op3 = 0);
	StateAccess(Opcode opcode, const MutableVal* target, const Val* op1,
			const Val* op2 = 0, const Val* op3 = 0);

	// For tables, the idx operand may be given as an index HashKey.
	// This is for efficiency. While we need to reconstruct the index
	// if we are actually going to serialize the access, we can at
	// least skip it if we don't.
	StateAccess(Opcode opcode, const ID* target, const HashKey* op1,
			const Val* op2 = 0, const Val* op3 = 0);
	StateAccess(Opcode opcode, const MutableVal* target, const HashKey* op1,
			const Val* op2 = 0, const Val* op3 = 0);

	~StateAccess();

	// Replays this access in the our environment.
	void Replay();

	void Describe(ODesc* d) const;

	bool Serialize(Serializer* s) const;
	static StateAccess* Unserialize(Serializer* s);

private:
	StateAccess()	{ target.id = 0; op1.val = op2 = op3 = 0; }
	void RefThem();

	bool CheckOld(ID* id, Val* index, Val* should, Val* is);
	bool CheckOldSet(ID* id, Val* index, bool should, bool is);

	Opcode opcode;
	union {
		ID* id;
		MutableVal* val;
	} target;

	union {
		Val* val;
		const HashKey* key;
	} op1;

	Val* op2;
	Val* op3;

	enum Type { TYPE_ID, TYPE_MVAL, TYPE_VAL, TYPE_KEY };
	Type target_type;
	Type op1_type;
};

#endif
