// $Id: Stats.h,v 1.4 2004/03/21 17:23:25 vern Exp $
//
// Classes that collect and report statistics.

#ifndef STATS_H
#define STATS_H

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>

class StatsLogger {
public:
	StatsLogger(BroFile* file, double interval);
	~StatsLogger();

	void Log();
	BroFile* File()	{ return file; }

	void SegmentProfile(const char* name, const Location* loc,
				double dtime, int dmem);

private:
	BroFile* file;
};

extern StatsLogger* stats_logger;
extern StatsLogger* segment_logger;

// A SegmentProfiler tracks how much CPU and memory is consumed
// across its lifetime.
//
// ### This needs porting to Linux.  It could also be improved by
// better efforts at measuring its own overhead.
class SegmentProfiler {
public:
	// The constructor takes some way of identifying the segment.
	SegmentProfiler(StatsLogger* arg_logger, const char* arg_name)
		{
		logger = arg_logger;
		if ( logger )
			{
			name = arg_name;
			loc = 0;
			Init();
			}
		}

	SegmentProfiler(StatsLogger* arg_logger, const Location* arg_loc)
		{
		logger = arg_logger;
		if ( logger )
			{
			name = 0;
			loc = arg_loc;
			Init();
			}
		}

	~SegmentProfiler()
		{
		if ( logger )
			Report();
		}

protected:
	void Init();
	void Report();

	StatsLogger* logger;
	const char* name;
	const Location* loc;
	struct rusage initial_rusage;
};

#endif
