# $Id: blaster.bro,v 1.1 2003/09/10 06:14:19 vern Exp $
#
# Identifies W32.Blaster-infected hosts by observing their scanning
# activity.

@load alert
@load site

# Which hosts have scanned which addresses via 135/tcp.
global w32b_scanned: table[addr] of set[addr] &write_expire = 5min;
global w32b_reported: set[addr] &persistent;

const W32B_port = 135/tcp;
const W32B_MIN_ATTEMPTS = 50 &redef;

redef enum Alert += {
	W32B_SOURCE_LOCAL,
	W32B_SOURCE_REMOTE,
};

event connection_attempt(c: connection)
	{
	if ( c$id$resp_p != W32B_port )
		return;

	local ip = c$id$orig_h;

	if ( ip in w32b_reported )
		return;

	if ( ip in w32b_scanned )
		{
		add (w32b_scanned[ip])[c$id$resp_h];

		if ( length(w32b_scanned[ip]) == W32B_MIN_ATTEMPTS )
			{
			if ( is_local_addr(ip) )
				ALERT([$alert=W32B_SOURCE_LOCAL, $conn=c,
					$msg=fmt("W32.Blaster local source: %s",
							ip)]);
			else
				ALERT([$alert=W32B_SOURCE_REMOTE, $conn=c,
					$msg=fmt("W32.Blaster remote source: %s",
							ip)]);

			add w32b_reported[ip];
			}
		}

	else
		{
		local s: set[addr];
		add s[ip];
		w32b_scanned[ip] = s;
		}
	}
