# $Id: remote.bro,v 1.4 2004/03/01 12:14:33 vern Exp $
#
# Connect to remote Bros and request some of their events.

@load alert

type Peer: record {
	events: pattern &optional;
	retry: interval &default = 0 secs;

	# If not set, no capture filter is send.
	# If set to "", the default cature filter is send.
	capture_filter: string &optional;

	# Take-over state from this host
	# (activated by loading hand-over.bro)
	hand_over: bool &default = F;
};

const remote_peers_clear: table[addr, port] of Peer &redef;
const remote_peers_ssl: table[addr, port] of Peer &redef;

# redef remote_peers_ssl += {
# 	[127.0.0.1, 47756/tcp] = [$events = /.*/, $retry = 60 secs]
# };

global remote_log = open_log_file("remote");

# If true, re-raise received alerts locally.
global report_remote_alerts = T &redef;

function build_capture_filter_index(ip: addr, p: port): string
	{
	return fmt("remote-%s-%d", ip, p );
	}

event remote_connection_established(ip: addr, p: port)
	{
	print remote_log, fmt("%.6f %s/%d Connection established.",
				current_time(), ip, p);

	local peer: Peer;
	if ( [ip, p] in remote_peers_clear )
		{
		peer = remote_peers_clear[ip, p];

		if ( peer?$events )
			request_remote_events(ip, p, peer$events);

		if ( peer?$capture_filter )
			if ( peer$capture_filter != "" )
				send_capture_filter(ip, p, peer$capture_filter);
			else
				send_capture_filter(ip, p, default_pcap_filter);
		}

	if ( [ip, p] in remote_peers_ssl )
		{
		peer = remote_peers_ssl[ip, p];

		if ( peer?$events )
			request_remote_events(ip, p, peer$events);

		if ( peer?$capture_filter )
			if ( peer$capture_filter != "" )
				send_capture_filter(ip, p, peer$capture_filter);
			else
				send_capture_filter(ip, p, default_pcap_filter);
		}
	}

event remote_connection_closed(ip: addr, p: port)
	{
	local i = build_capture_filter_index(ip, p);
	if ( i in capture_filters )
		{
		delete capture_filters[i];
		update_default_pcap_filter();
		}

	print remote_log, fmt("%.6f %s/%d Connection closed.",
				current_time(), ip, p);
	}

event remote_connection_error(ip: addr, p: port, reason: string)
	{
	print remote_log, fmt("%.6f %s/%d Error: %s",
				current_time(), ip, p, reason);
	}

event remote_state_inconsistency(id: string, expected_old: string, real_old: string)
	{
	print remote_log, fmt("%.6f State inconsistency: %s should be %s but is %s",
				current_time(), id, expected_old, real_old);
	}

event bro_init()
	{
	set_buf(remote_log, F);

	for ( [ip, p] in remote_peers_clear )
		if ( p != 0/tcp )
			connect_clear(ip, p, remote_peers_clear[ip, p]$retry);

	for ( [ip, p] in remote_peers_ssl )
		if ( p != 0/tcp )
			connect_ssl(ip, p, remote_peers_ssl[ip, p]$retry);
	}

event alert_action(a: alert_info, action: AlertAction)
	{
	# Forward remote alert to our local alarm system.
	if ( report_remote_alerts && is_remote_event() )
		ALERT(a);
	}
